@@ -1,181 +1,21 @@
 
 import sys, os
+from pathlib import Path
+from copy import deepcopy, copy
 import humpack as hp
 import io, yaml, json
 from collections import defaultdict, OrderedDict
 from c3linearize import linearize
 
-from omnibelt import load_yaml, get_printer
+from omnibelt import save_yaml, load_yaml, get_printer
 
-from .util import primitives
-from .errors import YamlifyError, MissingConfigError, UnknownActionError, InvalidKeyError
-from .external import find_config_path
-from .registry import create_component, _appendable_keys, Component
+from .util import primitives, global_settings, configurize, pythonize, ConfigurizeFailed
+from .errors import PythonizeError, MissingParameterError, UnknownActionError, InvalidKeyError
+# from .registry import create_component, _appendable_keys, Component
 
-nones = {'None', 'none', '_none', '_None', 'null', 'nil', }
 
 prt = get_printer(__name__)
 
-def load_config_from_path(path, process=True):
-	'''
-	Load the yaml file and transform data to a config object
-	
-	Generally, ``get_config`` should be used instead of this method
-	
-	:param path: must be the full path to a yaml file
-	:param process: if False, the loaded yaml data is passed without converting to a config object
-	:return: loaded data from path (usually as a config object)
-	'''
-	# path = find_config_path(path)
-	data = load_yaml(path)
-	
-	if data is None:
-		data = {}
-	
-	if process:
-		return configurize(data)
-	return data
-
-def process_single_config(data, process=True, parents=None):  # data can either be an existing config or a path to a config
-	'''
-	This loads the data (if a path or name is provided) and then checks for parents and loads those as well
-	
-	Generally, ``get_config`` should be used instead of this method
-	
-	:param data: config name or path or raw data (dict/list) or config object
-	:param process: configurize loaded data
-	:param parents: if None, no parents are loaded, otherwise it must be a dict where the keys are the absolute paths to the config (yaml) file and values are the loaded data
-	:return: loaded data (as a config object or raw)
-	'''
-	
-	if isinstance(data, str):
-		data = find_config_path(data)
-		data = load_config_from_path(data, process=process)
-	
-	if parents is not None and 'parents' in data:
-		todo = []
-		for parent in data['parents']:  # prep new parents
-			# ppath = _config_registry[parent] if parent in _config_registry else parent
-			ppath = find_config_path(parent)
-			if ppath not in parents:
-				todo.append(ppath)
-				parents[ppath] = None
-		for ppath in todo:  # load parents
-			parents[ppath] = process_single_config(ppath, parents=parents)
-	
-	return data
-
-def merge_configs(configs, parent_defaults=True):
-	'''
-	configs should be ordered from oldest to newest (ie. parents first, children last)
-	
-	This is an internal method used by ``get_config()`` and should generally not be called manually.
-	'''
-	
-	if not len(configs):
-		return ConfigDict()
-	
-	child = configs.pop()
-	merged = merge_configs(configs, parent_defaults=parent_defaults)
-	
-	# load = child.load if 'load' in child else None
-	merged.update(child)
-	
-	return merged
-
-
-def get_config(*contents, **manual):  # Top level function
-	'''
-	Top level function for users. This is the best way to load/create a config object.
-
-	All parent config (registered names or paths) that should be loaded
-	must precede any manual entries, and will be loaded in reverse order (like python class inheritance).
-	
-	If the key ``_history_key`` is specified and not :code:`None`, a flattened list of all parents of
-	this config is pushed to the given key.
-	
-	:param contents: registered configs or paths or manual entries (like in terminal)
-	:param manual: specify parameters manually as key value pairs
-	:return: config object
-	'''
-	root = ConfigDict()
-	if len(contents) + len(manual) == 0:
-		return root
-
-	reg = []
-	terms = {**manual}
-	allow_reg = True
-	waiting_key = None
-	
-	for term in contents:
-		
-		if term.startswith('--'):
-			allow_reg = False
-			if waiting_key is not None:
-				terms[waiting_key] = True
-			waiting_key = term[2:]
-		
-		elif waiting_key is not None:
-			terms[waiting_key] = process_raw_argv(term)
-			waiting_key = None
-			
-		elif allow_reg:
-			reg.append(term)
-		
-		else:
-			raise Exception(f'Parsing error: {term} in {contents}')
-		
-	if waiting_key is not None:
-		terms[waiting_key] = True
-		
-	root.update(configurize(terms))
-	
-	if len(reg) == 0:
-		return root
-
-	root['parents'] = ConfigList(data=reg + (list(root['parents']) if 'parents' in root else []))
-
-	parents = {}
-	
-	root = process_single_config(root, parents=parents)
-
-	pnames = []
-	if len(parents):  # topo sort parents
-		
-		# TODO: maybe clean up?
-		
-		root_id = ' root'
-		src = defaultdict(list)
-		
-		names = {find_config_path(p): p for p in root['parents']} if 'parents' in root else {}
-		src[root_id] = list(names.keys())
-		
-		for n, data in parents.items():
-			connections = {find_config_path(p): p for p in data['parents']} if 'parents' in data else {}
-			names.update(connections)
-			src[n] = list(connections.keys())
-		
-		order = linearize(src, heads=[root_id], order=True)[root_id]
-		
-		pnames = [names[p] for p in order[1:]]
-		order = [root] + [parents[p] for p in order[1:]]
-		
-		# for analysis, record the history of all loaded parents
-		order = list(reversed(order))
-	
-	else:  # TODO: clean up
-		order = [root]
-	
-	root = merge_configs(order,)
-	
-	include_history = root.pull('_history_key', None, silent=True)
-	if include_history is not None:
-		root.push(include_history, pnames, silent=True)
-	
-	root.push('parents', '_x_', silent=True)
-	
-	return root
-
 
 
 
@@ -194,13 +34,17 @@ class Config_Printing:
 			_printing_instance.unit = ' > '
 			_printing_instance.style = '| '
 			_printing_instance.silent = False
-			
+			_printing_instance.src = None
+		
 		return _printing_instance
 	def __repr__(self):
 		return f'ConfigPrinting[{hex(id(self))}]'
 	def __str__(self):
 		return f'ConfigPrinting'
 	
+	# def set_src(self, src):
+	# 	self.src = src
+	
 	def inc_indent(self):
 		self.level += 1
 	def dec_indent(self):
@@ -233,7 +77,8 @@ class Config_Printing:
 	               silent=False):
 		indent = self.level * self.unit
 		style = self.style
-		prefix = style + indent
+		src = '' if self.src is None else f'({self.src}) '
+		prefix = style + src + indent
 		
 		msg = raw.replace('\n', '\n' + prefix)
 		if not self.is_new_line:
@@ -271,7 +116,8 @@ class ConfigType(hp.Transactionable):
 	
 	'''
 
-	def __init__(self, parent=None, silent=False, printer=None, prefix=None, safe_mode=False,
+	def __init__(self, parent=None, silent=False, printer=None,
+	             prefix=None, safe_mode=False, project=None,
 	             data=None):
 		'''
 		Generally it should not be necessary to create a ConfigType manually, instead use ``get_config()``.
@@ -280,6 +126,7 @@ class ConfigType(hp.Transactionable):
 		:param silent: don't print ``pull``s and ``push``es
 		:param printer: printer object to handle printing messages
 		:param prefix: initial prefix used for printing
+		:param project: project this config responds to
 		:param safe_mode: don't save created component instances, unless during a transaction
 		:param data: raw parameters to immediately add to this config
 		'''
@@ -294,13 +141,31 @@ class ConfigType(hp.Transactionable):
 		self.__dict__['_hidden_prefix'] = prefix.copy()
 		self.__dict__['_safe_mode'] = safe_mode
 		
+		self.__dict__['_project'] = None
+		
 		self.set_parent(parent)
 		# self._set_silent(silent)
 
+		if project is not None:
+			self.set_project(project)
+
 		super().__init__()
 		
 		if data is not None:
 			self.update(data)
+
+	def __deepcopy__(self, memodict={}):
+		raise NotImplementedError
+
+	def __copy__(self):
+		raise NotImplementedError
+
+	def copy(self):
+		return copy(self)
+
+	@classmethod
+	def convert(cls, data, recurse):
+		return cls(data=[recurse(x) for x in data])
 		
 	def sub(self, item):
 		'''
@@ -335,7 +200,7 @@ class ConfigType(hp.Transactionable):
 			raise NotImplementedError
 
 	def _record_action(self, action, suffix=None, val=None, silent=False, obj=None,
-	                   defaulted=False, pushed=False, _entry=False):
+	                   defaulted=False, pushed=False, _entry=False, _raw=False):
 		'''
 		Internal function to manage printing out messages after various actions have been taken with this config.
 		
@@ -376,7 +241,7 @@ class ConfigType(hp.Transactionable):
 			mods = val.pull('_mod', None, silent=True)
 			
 			mod_info = ''
-			if mods is not None:
+			if mods is not None and len(mods):
 				if not isinstance(mods, (list, tuple)):
 					mods = mods,
 				
@@ -390,8 +255,7 @@ class ConfigType(hp.Transactionable):
 			
 			# head = '' if suffix is None else f' {name}'
 			head = f' {name}'
-			out = printer.log_record(f'{action.upper()}{head} '
-			                                               f'(type={cmpn_type}){mod_info}{origins}{end}',
+			out = printer.log_record(f'{action.upper()}{head} (type={cmpn_type}){mod_info}{origins}{end}',
 			                       silent=silent)
 			if action == 'creating':
 				printer.inc_indent()
@@ -419,7 +283,7 @@ class ConfigType(hp.Transactionable):
 			return ''
 			return printer.log_record(f'=> id={hex(id(obj))}', silent=silent)
 
-		pval = None if val is None else repr(val)
+		pval = None if val is None else (f'[{type(val)}]' if _raw else repr(val))
 		
 		if action == 'entry': # when pulling dict/list
 			assert suffix is not None, 'no suffix provided'
@@ -434,7 +298,7 @@ class ConfigType(hp.Transactionable):
 		
 		raise UnknownActionError(action)
 		
-	def pull(self, item, *defaults, silent=False, ref=False, no_parent=False, as_iter=False):
+	def pull(self, item, *defaults, silent=False, ref=False, no_parent=False, as_iter=False, raw=False):
 		'''
 		Top-level function to get parameters from the config object (including automatically creating components)
 
@@ -447,9 +311,10 @@ class ConfigType(hp.Transactionable):
 		:return: processed value of the parameter (or default if ``item`` is not found, or raises a ``MissingConfigError`` if not found)
 		'''
 		self._reset_prefix()
-		return self._pull(item, *defaults, silent=silent, ref=ref, no_parent=no_parent, as_iter=as_iter, _origin=self)
+		return self._pull(item, *defaults, silent=silent, ref=ref, no_parent=no_parent, as_iter=as_iter,
+		                  _origin=self, _raw=raw)
 
-	def pull_self(self, name=None, silent=False, as_iter=False):
+	def pull_self(self, name=None, silent=False, as_iter=False, raw=False, ref=False):
 		'''
 		Process self as a value being pulled.
 		
@@ -459,10 +324,11 @@ class ConfigType(hp.Transactionable):
 		:return: the processed value of self
 		'''
 		self._reset_prefix()
-		return self._process_val(name, self, silent=silent, reuse=False, is_self=True, as_iter=as_iter, _origin=self)
+		return self._process_val(name, self, silent=silent, reuse=ref, is_self=True, as_iter=as_iter,
+		                         _origin=self, _raw=raw)
 
 	def _pull(self, item, *defaults, silent=False, ref=False, no_parent=False, as_iter=False,
-	          _defaulted=False, _origin=None):
+	          _defaulted=False, _origin=None, _raw=False):
 		'''
 		Internal pull method, should generally not be called manually (unless you know what you're doing)
 		
@@ -474,6 +340,7 @@ class ConfigType(hp.Transactionable):
 		:param as_iter: return the value as an iterator (only for dicts/lists)
 		:param _defaulted: flag that this value was once a provided default (used for printing)
 		:param _origin: reference to the original config node that was pulled (some pulls require returing to origin)
+		:param _raw: return unprocessed value
 		:return: processed value found at ``item`` or a processed default value
 		'''
 		
@@ -493,8 +360,21 @@ class ConfigType(hp.Transactionable):
 		if no_parent and byparent:
 			defaulted = True
 		if defaulted:
+			try:
+				if '__origin_key' in self: # cousins
+					origin = self['__origin_key']
+					if origin is not None:
+						parent = self.get_parent()
+						if parent is not None:
+							grandparent = parent.get_parent()
+							if grandparent is not None:
+								return grandparent._pull((origin, item), silent=silent, _defaulted=defaulted or _defaulted,
+							                        as_iter=as_iter, ref=ref, _origin=_origin, _raw=_raw)
+			except MissingParameterError:
+				pass
+			
 			if len(defaults) == 0:
-				raise MissingConfigError(item)
+				raise MissingParameterError(item)
 			val, *defaults = defaults
 			line = []
 		else:
@@ -503,15 +383,15 @@ class ConfigType(hp.Transactionable):
 		if len(line) and not isinstance(val, ConfigType):
 			defaulted = True
 			if len(defaults) == 0:
-				raise MissingConfigError(item)
+				raise MissingParameterError(item)
 			val, *defaults = defaults
 
-		if defaulted: # try again with new value
+		if defaulted and _origin is not None: # try again with new value
 
 			_origin._reset_prefix()
 			
 			val = _origin._process_val(item, val, *defaults, silent=silent, defaulted=defaulted or _defaulted,
-			                        as_iter=as_iter, reuse=ref, _origin=_origin)
+			                        as_iter=as_iter, reuse=ref, _origin=_origin, _raw=_raw)
 
 		elif len(line): # child pull
 			if not isinstance(val, ConfigType):
@@ -519,20 +399,20 @@ class ConfigType(hp.Transactionable):
 			
 			val._set_prefix(self.get_prefix() + [item])
 			out = val._pull(line, *defaults, silent=silent, ref=ref, no_parent=no_parent, as_iter=as_iter,
-			               _defaulted=_defaulted, _origin=_origin)
+			               _defaulted=_defaulted, _origin=_origin, _raw=_raw)
 			
 			val = out
 			
-		elif byparent: # parent pull
+		elif byparent and not item.startswith('_'): # parent pull
 			parent = self.get_parent()
 
 			parent._set_prefix(self.get_prefix() + [''])
 			val = parent._pull((item, *line), *defaults, silent=silent, ref=ref, no_parent=no_parent, as_iter=as_iter,
-			                   _origin=_origin)
+			                   _origin=_origin, _raw=_raw)
 			
 		else: # process val from me/defaults
 			val = self._process_val(item, val, *defaults, silent=silent, defaulted=defaulted or _defaulted,
-			                        as_iter=as_iter, reuse=ref, _origin=_origin)
+			                        as_iter=as_iter, reuse=ref, _origin=_origin, _raw=_raw)
 			
 			if type(val) in {list, set}: # TODO: a little heavy handed
 				val = tuple(val)
@@ -540,7 +420,8 @@ class ConfigType(hp.Transactionable):
 		return val
 
 	def _process_val(self, item, val, *defaults, silent=False,
-	                 reuse=False, is_self=False, as_iter=False, _origin=None, **record_flags):
+	                 reuse=False, is_self=False, as_iter=False, _origin=None, _raw=False,
+	                 **record_flags):
 		'''
 		This is used by ``pull()`` to process the recovered value and print the correct message if ``not silent``
 		
@@ -557,19 +438,25 @@ class ConfigType(hp.Transactionable):
 		'''
 		
 		if as_iter and isinstance(val, ConfigType):
-			
+
 			obj_type = 'list' if isinstance(val, list) else 'dict'
-			
-			self._record_action(f'iter-{obj_type}', suffix=item, val=val, silent=silent, **record_flags)
-			
-			itr = Config_Iter(val, val)
-			
-			return itr
+
+			if obj_type == 'list' or '_type' not in val or not val.pull('_type', silent=True).startswith('iter'):
+
+				self._record_action(f'iter-{obj_type}', suffix=item, val=val, silent=silent, **record_flags)
+
+				if _raw:
+					return val
+
+				itr = ConfigIter(val, val)
+
+				return itr
 		
-		if isinstance(val, ConfigDict):
-			if '_type' in val:
-				
-				# val.push('__origin_key', item, silent=True)
+		if isinstance(val, ConfigDict) and not _raw:
+			
+			val.push('__origin_key', item, silent=True)
+			typ = val._pull('_type', None, silent=True)
+			if typ is not None:
 				
 				if reuse and '__obj' in val:
 					# print('WARNING: would usually reuse {} now, but instead creating a new one!!')
@@ -585,11 +472,11 @@ class ConfigType(hp.Transactionable):
 					
 					past = self._get_silent()
 					with self.silenced(silent or past):
-						cmpn = create_component(val)
-					
+						cmpn = self.get_project().create_component(val)
+
 					if not self.in_safe_mode() or self.in_transaction():
 						if item is not None and len(item) and not is_self:
-							self[item]['__obj'] = cmpn
+							val['__obj'] = cmpn
 						else:
 							self['__obj'] = cmpn
 					
@@ -602,6 +489,8 @@ class ConfigType(hp.Transactionable):
 
 			else:
 				
+				val.push('__origin_key', '_x_', silent=True)
+				
 				self._record_action('pull-dict', suffix=item, val=val, silent=silent, **record_flags)
 				terms = {}
 				for k, v in val.items():  # WARNING: pulls all entries in dict
@@ -611,7 +500,7 @@ class ConfigType(hp.Transactionable):
 				val = terms
 
 
-		elif isinstance(val, ConfigList):
+		elif isinstance(val, ConfigList) and not _raw:
 			
 			self._record_action('pull-list', suffix=item, val=val, silent=silent, **record_flags)
 			terms = []
@@ -626,7 +515,7 @@ class ConfigType(hp.Transactionable):
 			
 			self._record_action('local-alias', suffix=item, val=alias, silent=silent, **record_flags)
 			
-			val = self._pull(alias, *defaults, silent=silent, _origin=_origin)
+			val = self._pull(alias, *defaults, silent=silent, _origin=_origin, as_iter=as_iter)
 			
 		elif isinstance(val, str) and val.startswith('<o>'): # origin alias (returns to origin to find alias)
 			alias = val[3:]
@@ -634,14 +523,28 @@ class ConfigType(hp.Transactionable):
 			self._record_action('origin-alias', suffix=item, val=alias, silent=silent, **record_flags)
 			
 			_origin._reset_prefix()
-			val = _origin._pull(alias, *defaults, silent=silent, _origin=_origin)
-			
+			val = _origin._pull(alias, *defaults, silent=silent, _origin=_origin, as_iter=as_iter)
+
+		elif isinstance(val, str) and val.startswith('<!>'): # copy alias (only for local aliases)
+			alias = val[3:]
+
+			self._record_action('copy-alias', suffix=item, val=alias, silent=silent, **record_flags)
+
+			val = self._pull(alias, *defaults, silent=silent, _origin=_origin, as_iter=as_iter, _raw=True)
+
+			# self[item] = deepcopy(val)
+			val = deepcopy(val)
+			self[item] = val
+
+			return self._process_val(item, val, silent=silent, _origin=_origin, as_iter=as_iter, _raw=_raw)
+
+
 		else:
-			self._record_action('pulled', suffix=item, val=val, silent=silent, **record_flags)
+			self._record_action('pulled', suffix=item, val=val, silent=silent, _raw=_raw, **record_flags)
 
 		return val
 	
-	def push(self, key, val, *_skip, silent=False, overwrite=True, no_parent=True, force_root=False):
+	def push(self, key, val, *_skip, silent=False, overwrite=True, no_parent=True, force_root=False, process=True):
 		'''
 		Set ``key`` with ``val`` in the config object, but pulls ``key`` first so that `val` is only set
 		if it is not found or ``overwrite`` is set to ``True``. It will return the current value of ``key`` after
@@ -657,9 +560,11 @@ class ConfigType(hp.Transactionable):
 		:return: current val of key (updated if written)
 		'''
 		self._reset_prefix()
-		return self._push(key, val, silent=silent, overwrite=overwrite, no_parent=no_parent, force_root=force_root)
+		return self._push(key, val, silent=silent, overwrite=overwrite, no_parent=no_parent,
+		                  force_root=force_root, process=process)
 	
-	def _push(self, key, val, silent=False, overwrite=True, no_parent=True, force_root=False, _origin=None):
+	def _push(self, key, val, silent=False, overwrite=True, no_parent=True, force_root=False,
+	          process=True, _origin=None):
 		'''
 		Set ``key`` with ``val`` in the config object, but pulls ``key`` first so that `val` is only set
 		if it is not found or ``overwrite`` is set to ``True``. It will return the current value of ``key`` after
@@ -689,7 +594,7 @@ class ConfigType(hp.Transactionable):
 		
 		if parent is not None and force_root:
 			return self.get_root().push((key, *line), val, silent=silent, overwrite=overwrite,
-			                   no_parent=no_parent)
+			                   no_parent=no_parent, process=process)
 		elif no_parent:
 			parent = None
 		
@@ -698,14 +603,15 @@ class ConfigType(hp.Transactionable):
 			
 			child._set_prefix(self.get_prefix() + [key])
 			
-			out = child._push(line, val, silent=silent, overwrite=overwrite, no_parent=True)
+			out = child._push(line, val, silent=silent, overwrite=overwrite, no_parent=True, process=process)
 			
 			return out
 		elif parent is not None and key in parent: # push parent
 
 			parent._set_prefix(self.get_prefix() + [''])
 			
-			out = parent._push((key, *line), val, silent=silent, overwrite=overwrite, no_parent=no_parent)
+			out = parent._push((key, *line), val, silent=silent, overwrite=overwrite,
+			                   no_parent=no_parent, process=process)
 			
 			return out
 			
@@ -715,7 +621,7 @@ class ConfigType(hp.Transactionable):
 			
 			child._set_prefix(self.get_prefix() + [key])
 			
-			out = child._push(line, val, silent=silent, overwrite=overwrite, no_parent=True)
+			out = child._push(line, val, silent=silent, overwrite=overwrite, no_parent=True, process=process)
 		
 			return out
 		
@@ -734,7 +640,8 @@ class ConfigType(hp.Transactionable):
 		self[key] = val
 		# val = self[key]
 		
-		val = self._process_val(key, val, silent=silent, pushed=True)
+		if process:
+			val = self._process_val(key, val, silent=silent, pushed=True)
 		
 		return val
 
@@ -747,13 +654,13 @@ class ConfigType(hp.Transactionable):
 		:return: raw "yamlified" data
 		'''
 
-		data = yamlify(self)
+		data = pythonize(self)
 
 		if path is not None:
-			if os.path.isdir(path):
-				path = os.path.join(path, 'config.yaml')
-			with open(path, 'w') as f:
-				yaml.dump(data, f)
+			path = Path(path)
+			if path.is_dir():
+				path = path / 'config.yaml'
+			save_yaml(data, path)
 			return path
 
 		return data
@@ -765,11 +672,14 @@ class ConfigType(hp.Transactionable):
 		
 		:return: iterator over all arguments in self
 		'''
-		return Config_Iter(self, self)
-	
+		return ConfigIter(self, self)
+
+	def replace_vals(self, replacements):
+		raise NotImplementedError
+
 	# region Silencing
 	
-	def _set_silent(self, silent=True):
+	def set_silent(self, silent=True):
 		self.__dict__['_printer'].silent = silent
 	
 	# self._silent_config_flag = silent
@@ -786,11 +696,11 @@ class ConfigType(hp.Transactionable):
 			self.prev = config._get_silent()
 		
 		def __enter__(self):
-			self.config._set_silent(self.setting)
+			self.config.set_silent(self.setting)
 			return self.config
 		
 		def __exit__(self, exc_type, exc_val, exc_tb):
-			self.config._set_silent(self.prev)
+			self.config.set_silent(self.prev)
 	
 	def silenced(self, setting=True):
 		return ConfigType._Silent_Config(self, setting=setting)
@@ -810,6 +720,9 @@ class ConfigType(hp.Transactionable):
 		'''Get parent (returns None if this is the root)'''
 		return self.__dict__['_parent']
 	
+	def set_process_id(self, name=None):
+		self._get_printer().src = name
+	
 	def get_root(self):
 		'''Gets the root config object (returns ``self`` if ``self`` is the root)'''
 		parent = self.get_parent()
@@ -843,6 +756,12 @@ class ConfigType(hp.Transactionable):
 	
 	# region Misc
 	
+	def set_project(self, project):
+		self.get_root().__dict__['_project'] = project
+	
+	def get_project(self):
+		return self.get_root().__dict__['_project']
+	
 	def set_safe_mode(self, safe_mode):
 		if self.is_root():
 			self.__dict__['_safe_mode'] = safe_mode
@@ -854,14 +773,6 @@ class ConfigType(hp.Transactionable):
 			return self.__dict__['_safe_mode']
 		return self.get_root().in_safe_mode()
 	
-	@staticmethod
-	def parse_argv(arg):
-		try:
-			return yaml.safe_load(io.StringIO(arg))
-		except:
-			pass
-		return arg
-	
 	def purge_volatile(self):
 		'''
 		Recursively remove any items where the key starts with "__"
@@ -876,6 +787,7 @@ class ConfigType(hp.Transactionable):
 		return f'{type(self).__name__}[id={hex(id(self))}]'
 	
 	def __str__(self):
+		return repr(self)
 		return f'<{type(self).__name__}>'
 	
 	# endregion
@@ -889,7 +801,7 @@ class ConfigType(hp.Transactionable):
 		if isinstance(key, (list, tuple)):
 			if len(key) == 1:
 				return self.__setitem__(key[0], value)
-			child = self.__getitem__(key[0])
+			child = self.get_nodefault(*key)
 			assert isinstance(child, ConfigType)
 			return child.__setitem__(key[1:], value)
 		
@@ -897,9 +809,10 @@ class ConfigType(hp.Transactionable):
 		
 		if isinstance(value, ConfigType):
 			value.set_parent(self)
-		return super().__setitem__(key, value)
+			# value.set_project(self.get_project())
+		return self._single_set(key, value)
 	
-	def __getitem__(self, item):
+	def __getitem__(self, item, *future):
 		
 		if isinstance(item, str) and '.' in item:
 			item = item.split('.')
@@ -908,7 +821,7 @@ class ConfigType(hp.Transactionable):
 			if len(item) == 1:
 				item = item[0]
 			else:
-				return self.__getitem__(item[0])[item[1:]]
+				return self.__getitem__(*item)[item[1:]]
 		
 		parent = self.get_parent()
 		
@@ -917,9 +830,9 @@ class ConfigType(hp.Transactionable):
 				and item[0] != '_':
 			return parent[item]
 		
-		return self._single_get(item)
+		return self._single_get(item, *future)
 	
-	def get_nodefault(self, item):
+	def get_nodefault(self, item, *future):
 		'''Get ``item`` without defaulting up the tree if not found.'''
 		
 		if isinstance(item, str) and '.' in item:
@@ -929,20 +842,37 @@ class ConfigType(hp.Transactionable):
 			if len(item) == 1:
 				item = item[0]
 			else:
-				return self.get_nodefault(item[0])[item[1:]]
+				return self.get_nodefault(*item)[item[1:]]
 		
-		return self._single_get(item)
+		return self._single_get(item, *future)
 	
-	def _single_get(self, item):
+	def _single_set(self, key, val):
+		return super().__setitem__(key, val)
+	
+	def _single_get(self, item, *context):
 		try:
 			val = super().__getitem__(item)
+			if val is EmptyElement and len(context):
+				raise KeyError(item)
 		except KeyError:
-			return self._missing_key(item)
+			return self._missing_key(item, *context)
 		
 		return val
 	
-	def _missing_key(self, key):
-		obj = self.__class__(parent=self)
+	def _missing_key(self, key, *context):
+		
+		cls = ConfigDict
+		
+		if len(context):
+			nxt = context[0]
+			try:
+				int(nxt)
+			except ValueError:
+				pass
+			else:
+				cls = ConfigList
+		
+		obj = cls(parent=self)
 		self.__setitem__(key, obj)
 		return obj
 	
@@ -993,6 +923,41 @@ class ConfigDict(ConfigType, hp.tdict):
 	NOTE: avoid setting keys that start with more than one underscore (especially '__obj')
 	(unless you really know what you are doing)
 	'''
+
+	def __deepcopy__(self, memodict={}):
+		new = self.__class__(data={k:deepcopy(v) for k,v in self.items() if not k.startswith('__')})
+		new.__dict__.update(self.__dict__)
+		return new
+
+	def __copy__(self):
+		new = self.__class__(data={k:v for k,v in self.items()})
+		new.__dict__.update(self.__dict__)
+		return new
+
+	def copy(self):
+		return copy(self)
+
+	def replace_vals(self, replacements):
+		for k,v in self.items():
+			if isinstance(v, primitives) and v in replacements:
+				self[k] = replacements[v]
+			elif isinstance(v, ConfigType):
+				v.replace_vals(replacements)
+
+	@classmethod
+	def convert(cls, data, recurse):
+		return cls(data={k: recurse(v) for k, v in data.items()})
+		# try:
+		# 	inds = [int(k) for k in data]
+		# 	if not len(inds):
+		# 		raise ValueError
+		# except ValueError:
+		# 	return cls(data={k:recurse(v) for k,v in data.items()})
+		# else:
+		# 	ls = [EmptyElement]*max(inds)
+		# 	for i, v in zip(inds, data.values()):
+		# 		ls[i] = recurse(v)
+		# 	return ConfigList(ls)
 	
 	def update(self, other):
 		'''
@@ -1008,7 +973,6 @@ class ConfigDict(ConfigType, hp.tdict):
 			exists = self.contains_nodefault(k)
 			if exists and '_x_' == v:  # reserved for deleting settings in parents
 				del self[k]
-			
 			elif exists and isconfig and \
 					(isinstance(v, self[k].__class__)
 					 or isinstance(self[k], v.__class__)):
@@ -1038,20 +1002,34 @@ class ConfigDict(ConfigType, hp.tdict):
 		return f'[{hex(id(self))}]{info}'
 	
 	def __str__(self):
+		return repr(self)
 		return '{{' + ', '.join(f'{k}' for k in self) + '}}'
 
 
-
+class EmptyElement:
+	pass
 
 class ConfigList(ConfigType, hp.tlist):
 	'''
 	List like node in the config.
 	'''
 
-	def __init__(self, *args, empty_fill_value=None, **kwargs):
+
+
+	def __init__(self, *args, empty_fill_value=EmptyElement, **kwargs):
 		super().__init__(*args, **kwargs)
 		self._empty_fill_value = empty_fill_value
 
+	def __deepcopy__(self, memodict={}):
+		new = self.__class__(data=[deepcopy(x) for x in self])
+		new.__dict__.update(self.__dict__)
+		return new
+
+	def __copy__(self):
+		new = self.__class__(data=[x for x in self])
+		new.__dict__.update(self.__dict__)
+		return new
+
 	def __repr__(self):
 		info = '[[' + ', '.join(f'{k}' for k in self) + ']]'
 		return f'[{hex(id(self))}]{info}'
@@ -1059,6 +1037,14 @@ class ConfigList(ConfigType, hp.tlist):
 	def __str__(self):
 		return '[[' + ', '.join(f'{k}' for k in self) + ']]'
 
+	def replace_vals(self, replacements):
+		for i, x in enumerate(self):
+			if isinstance(x, primitives) and x in replacements:
+				self[i] = replacements[x]
+			elif isinstance(x, ConfigType):
+				x.replace_vals(replacements)
+
+
 	def purge_volatile(self):
 		'''
 		Recursively remove any items where the key starts with "__"
@@ -1089,16 +1075,32 @@ class ConfigList(ConfigType, hp.tlist):
 				self.append(x)
 			elif isconfig and (isinstance(x, self[i].__class__) or isinstance(self[i], x.__class__)):
 				self[i].update(x)
-			else:
+			elif x is not EmptyElement:
 				self[i] = x
 	
-	def _single_get(self, item):
-		try:
-			idx = self._str_to_int(item)
-		except InvalidKeyError:
-			idx = None
+
+	def _single_set(self, key, val):
+		if key == '_':  # append to end of the list
+			key = len(self)
+		key = self._str_to_int(key)
+		if key >= len(self):
+			self.extend([self._empty_fill_value] * (key - len(self) + 1))
+		return super()._single_set(key, val)
+	
+	def _single_get(self, item, *context):
 		
-		return super()._single_get(item if idx is None else idx)
+		if isinstance(item, slice):
+			return super(ConfigType, self).__getitem__(item)
+		
+		if item == '_':  # append to end of the list
+			item = len(self)
+		
+		item = self._str_to_int(item)
+		
+		if item >= len(self):
+			self.extend([self._empty_fill_value] * (item - len(self) + 1))
+			
+		return super()._single_get(item, *context)
 
 	def push(self, first, *rest, overwrite=True, **kwargs):
 		'''
@@ -1119,16 +1121,23 @@ class ConfigList(ConfigType, hp.tlist):
 		self.append(None)
 		return super().push(key, val, *rest, overwrite=True, **kwargs) # note that *rest will have no effect
 		
-	def __setitem__(self, key, value):
-
-		if key == '_': # append to end of the list
-			return self.__setitem__(len(self), value)
-
-		idx = self._str_to_int(key)
-		
-		if idx >= len(self):
-			self.extend([self._empty_fill_value]*(idx-len(self)+1))
-		return super().__setitem__(idx, value)
+	# def __setitem__(self, item, value):
+	#
+	# 	if isinstance(item, str) and '.' in item:
+	# 		item = item.split('.')
+	#
+	# 	if isinstance(item, (tuple, list)):
+	# 		key = item[0]
+	# 	else:
+	# 		key = item
+	#
+	#
+	# 	if isinstance(item, (tuple,list)) and len(item) > 1:
+	# 		item[0] = key
+	# 	else:
+	# 		item = key
+	#
+	# 	return super().__setitem__(item, value)
 		
 		
 	def contains_nodefault(self, item):
@@ -1145,6 +1154,7 @@ class ConfigList(ConfigType, hp.tlist):
 		super().append(item)
 		if isinstance(item, ConfigType):
 			item.set_parent(self)
+			# item.set_project(self.get_project())
 		
 	def extend(self, item):
 		super().extend(item)
@@ -1152,25 +1162,25 @@ class ConfigList(ConfigType, hp.tlist):
 		for x in item:
 			if isinstance(x, ConfigType):
 				x.set_parent(self)
+				# x.set_project(self.get_project())
 
-@Component('iter')
-class Config_Iter:
+
+class ConfigIter:
 	'''
 	Iterate through a list of parameters, processing each item lazily,
 	ie. only when it is iterated over (with ``next()``)
 	'''
-
-	def __init__(self, origin, elements=None):
+	
+	def __init__(self, origin, elements=None, auto_pull=True, include_key=None, reversed=False):
 		'''
 		Can be used as a component or created manually (by providing the ``elements`` argument explicitly)
-		
+
 		For dicts, this will behave like ``.items()``, ie. for each entry in the dict it will return
 		a tuple of the key and value.
-		
+
 		:param origin: config object where the iterator info is
 		:param elements: manually provided elements to iterate over (uses contents of "_elements" in ``origin`` if not provided)
 		'''
-		self._idx = 0
 		# self._name = config._ident
 		# assert '_elements' in config, 'No elements found'
 		if elements is None:
@@ -1178,106 +1188,100 @@ class Config_Iter:
 		self._elms = elements
 		
 		self._keys = [k for k in self._elms.keys()
-		              if k not in {'_elements', '_mod', '_type', '__obj'}
-					  and self._elms[k] != '__x__'] \
+		              if k not in {'_elements', '_mod', '_type', '__obj', '__origin_key'}
+		              and self._elms[k] != '__x__'] \
 			if isinstance(self._elms, dict) else None
+		self._include_key = include_key if include_key is not None else self._keys is not None
 		self._prefix = origin.get_prefix().copy()
-
+		
+		self._reversed = False
+		self._idx = 0
+		self.set_reversed(reversed)
+		self.set_auto_pull(auto_pull)
+	
 	def __len__(self):
 		'''Returns the remaining length of this iterator instance'''
 		return len(self._elms if self._keys is None else self._keys) - self._idx
-
+	
 	def _next_idx(self):
 		'''Find the next index or key'''
 		
 		if self._keys is None:
-			if not self._elms.contains_nodefault(self._idx):
+			if 0 > self._idx >= len(self._elms):
 				raise StopIteration
 			return str(self._idx)
+			# if not self._elms.contains_nodefault(self._idx):
+			# 	raise StopIteration
+			# return str(self._idx)
 		
-		while self._idx < len(self._elms):
+		while 0 <= self._idx < len(self._elms):
 			idx = self._keys[self._idx]
 			
 			if self._elms.contains_nodefault(idx):
 				return idx
-			self._idx += 1
+			self._idx += (-1)**self._reversed
 		
 		raise StopIteration
-		
+	
 	def view(self):
 		'''Returns the next object without processing the item, may throw a StopIteration exception'''
 		idx = self._next_idx()
-		obj = self._elms[idx]
+		obj = self._elms.pull(idx, raw=True, silent=True)
+		# obj = self._elms[idx]
 		if isinstance(obj, ConfigType):
-			return self._elms.sub(idx)
+			obj.push('_iter_key', idx, silent=True)
+			
+		if isinstance(obj, global_settings['config_type']):
+			obj = self._elms.sub(idx)
+		return (idx, obj) if self._include_key else obj
+
+	def step(self):
+		obj = self.view()
+		self._idx += (-1)**self._reversed
 		return obj
 
+	def set_auto_pull(self, auto=True):
+		self._auto_pull = auto
+
+	def set_reversed(self, reversed=True):
+		self._reversed = reversed
+		if reversed:
+			self._idx = len(self)-1
+
+	def has_next(self):
+		return (self._reversed and self._idx >= 0) or (not self._reversed and self._idx < len(self._elms))
+
 	def __next__(self):
-
-		if len(self._elms) == self._idx:
+		
+		if not self.has_next():
 			raise StopIteration
-		
-		idx = self._next_idx()
-
-		self._elms._prefix = self._prefix
-		
-		obj = self._elms._pull(idx)
-		
-		self._idx += 1
-		if self._keys is None:
-			return obj
-		return idx, obj
 
+		obj = self.step()
+		key, val = obj if self._include_key else (None,obj)
+		if isinstance(val, global_settings['config_type']):
+			val = val.pull_self(raw=not self._auto_pull, silent=not self._auto_pull)
+		return (key,val) if self._include_key else val
+	
 	def __iter__(self):
 		return self
 
+nones = {'None', 'none', '_none', '_None', 'null', 'nil', }
 
-def configurize(data):
-	'''
-	Transform data container to use config objects (ConfigDict/ConfigList)
-
-	:param data: dict/list data
-	:return: deep copy of data using ConfigDict/ConfigList
-	'''
-	if isinstance(data, ConfigType):
-		return data
-	if isinstance(data, dict):
-		return ConfigDict(data={k: configurize(v) for k, v in data.items()})
-	if isinstance(data, (list, set)):
-		return ConfigList(data=[configurize(x) for x in data])
-	if isinstance(data, str) and data in nones:
+def configurize_nones(s, recurse):
+	if s in nones:
 		return None
-	return data
-
-
-def yamlify(data):  # TODO: allow adding yamlify rules for custom objects
-	'''
-	Transform data container into regular dicts/lists to export to yaml file
-
-	:param data: Config object
-	:return: deep copy of data using dict/list
-	'''
-	# if data is None:
-	# 	return '_None'
-	if data is None or isinstance(data, primitives):
-		return data
-	if isinstance(data, dict):
-		return {k: yamlify(v) for k, v in data.items() if not k.startswith('__')}
-	if isinstance(data, (list, tuple, set)):
-		return [yamlify(x) for x in data]
-	
-	raise YamlifyError(data)
-
-
-# _config_type = ConfigType
-# _config_dict = ConfigDict
-# _config_list = ConfigList
-
-def process_raw_argv(arg):
-	'''
-	This is the preferred way to parse arguments for the config
-	(especially arguments specified through the terminal)
-	'''
-	return ConfigType.parse_argv(arg)
+	raise ConfigurizeFailed
+
+global_settings.update({
+	'config_type': ConfigType,
+	'config_converters': OrderedDict([
+		(str, configurize_nones),
+		(dict, (False, ConfigDict.convert)),
+		(OrderedDict, (False, ConfigDict.convert)),
+		(list, ConfigList.convert),
+		# (tuple, ConfigList.convert),
+		(set, ConfigList.convert),
+	]),
+})
 
 
