@@ -5,10 +5,14 @@ import traceback
 from omnibelt import get_printer
 
 from .errors import ConfigNotFoundError
-from .config import get_config
+
+from .top import get_config, find_config
+from .decorators import Component
+
 from .modes import Run_Mode
 from .rules import Meta_Rule
-from .registry import Component
+
+
 
 DEBUG_CODE = 'd'
 DEBUG_NAME = 'debug'
@@ -30,15 +34,15 @@ def debug_rule(meta, config):
 	if debug:
 		meta.push('_type', DEBUG_MODE_NAME, silent=True)
 		print('Switched to debug run mode')
-	
+		
 		try:
-			debug = get_config(DEBUG_NAME)
+			find_config(DEBUG_NAME)
 		except ConfigNotFoundError:
 			if not meta.pull('silent', False, silent=True):
 				prt.warning('No config "debug" was found')
 		else:
-			config.update(debug)
-		
+			config.update(get_config(DEBUG_NAME))
+	
 		meta.push('debug', False, silent=True) # to prevent running this rule multiple times for the same config
 
 	return config
